@@ -1,7 +1,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2017-2021 TwitchIO
+Copyright (c) 2017-present TwitchIO
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -28,7 +28,7 @@ import warnings
 import logging
 import traceback
 import sys
-from typing import Union, Callable, List, Optional, Tuple, Any
+from typing import Union, Callable, List, Optional, Tuple, Any, Coroutine, Dict
 
 from twitchio.errors import HTTPException
 from . import models
@@ -60,6 +60,9 @@ class Client:
         The event loop the client will use to run.
     heartbeat: Optional[float]
         An optional float in seconds to send a PING message to the server. Defaults to 30.0.
+    retain_cache: Optional[bool]
+        An optional bool that will retain the cache if PART is received from websocket when True.
+        It will still remove from cache if part_channels is manually called. Defaults to True.
 
     Attributes
     ------------
@@ -75,6 +78,7 @@ class Client:
         initial_channels: Union[list, tuple, Callable] = None,
         loop: asyncio.AbstractEventLoop = None,
         heartbeat: Optional[float] = 30.0,
+        retain_cache: Optional[bool] = True,
     ):
 
         self.loop: asyncio.AbstractEventLoop = loop or asyncio.get_event_loop()
@@ -89,10 +93,13 @@ class Client:
             loop=self.loop,
             initial_channels=initial_channels,
             heartbeat=heartbeat,
+            retain_cache=retain_cache,
         )
 
         self._events = {}
         self._waiting: List[Tuple[str, Callable[[...], bool], asyncio.Future]] = []
+        self.registered_callbacks: Dict[Callable, str] = {}
+        self._closing: Optional[asyncio.Event] = None
 
     @classmethod
     def from_client_credentials(
@@ -116,7 +123,7 @@ class Client:
 
         Parameters
         ------------
-        client_id: :class`str`
+        client_id: :class:`str`
 
         client_secret: :class:`str`
             An application Client Secret used to generate Access Tokens automatically.
@@ -139,6 +146,7 @@ class Client:
         )  # The only reason we're even creating this is to avoid attribute errors
         self._events = {}
         self._waiting = []
+        self.registered_callbacks = {}
         return self
 
     def run(self):
@@ -152,7 +160,9 @@ class Client:
         except KeyboardInterrupt:
             pass
         finally:
-            self.loop.run_until_complete(self.close())
+            if not self._closing.is_set():
+                self.loop.run_until_complete(self.close())
+
             self.loop.close()
 
     async def start(self):
@@ -167,14 +177,17 @@ class Client:
             )
         try:
             await self.connect()
+            await self._closing.wait()
         finally:
-            await self.close()
+            if not self._closing.is_set():
+                await self.close()
 
     async def connect(self):
         """|coro|
 
         Connects to the twitch IRC server
         """
+        self._closing = asyncio.Event()
         await self._connection._connect()
 
     async def close(self):
@@ -182,6 +195,7 @@ class Client:
 
         Cleanly disconnects from the twitch IRC server
         """
+        self._closing.set()
         await self._connection._close()
 
     def run_event(self, event_name, *args):
@@ -229,7 +243,7 @@ class Client:
             raise ValueError("Event callback must be a coroutine")
 
         event_name = name or callback.__name__
-        callback._event = event_name  # used to remove the event
+        self.registered_callbacks[callback] = event_name
 
         if event_name in self._events:
             self._events[event_name].append(callback)
@@ -238,11 +252,13 @@ class Client:
             self._events[event_name] = [callback]
 
     def remove_event(self, callback: Callable) -> bool:
-        if not hasattr(callback, "_event"):
+        event_name = self.registered_callbacks.get(callback)
+
+        if event_name is None:
             raise ValueError("Event callback is not a registered event")
 
-        if callback in self._events[callback._event]:
-            self._events[callback._event].remove(callback)
+        if callback in self._events[event_name]:
+            self._events[event_name].remove(callback)
             return True
 
         return False
@@ -283,9 +299,20 @@ class Client:
         fut = self.loop.create_future()
         tup = (event, predicate, fut)
         self._waiting.append(tup)
-        values = await asyncio.wait_for(fut, timeout, loop=self.loop)
+        values = await asyncio.wait_for(fut, timeout)
         return values
 
+    def wait_for_ready(self) -> Coroutine[Any, Any, bool]:
+        """|coro|
+
+        Waits for the underlying connection to finish startup
+
+        Returns
+        --------
+        :class:`bool` The state of the underlying flag. This will always be ``True``
+        """
+        return self._connection.is_ready.wait()
+
     @id_cache()
     def get_channel(self, name: str) -> Optional[Channel]:
         """Retrieve a channel from the cache.
@@ -308,9 +335,20 @@ class Client:
 
             return Channel(name=name, websocket=self._connection)
 
-    async def join_channels(self, channels: Union[List[str], Tuple[str]]):
+    async def part_channels(self, channels: Union[List[str], Tuple[str]]):
         """|coro|
 
+        Part the specified channels.
+
+        Parameters
+        ------------
+        channels: Union[List[str], Tuple[str]]
+            The channels in either a list or tuple form to part.
+        """
+        await self._connection.part_channels(*channels)
+
+    async def join_channels(self, channels: Union[List[str], Tuple[str]]):
+        """|coro|
 
         Join the specified channels.
 
@@ -336,6 +374,11 @@ class Client:
         """The IRC bots nick."""
         return self._http.nick or self._connection.nick
 
+    @property
+    def user_id(self):
+        """The IRC bot user id."""
+        return self._http.user_id or self._connection.user_id
+
     def create_user(self, user_id: int, user_name: str) -> PartialUser:
         """
         A helper method to create a :class:`twitchio.PartialUser` from a user id and user name.
@@ -389,7 +432,6 @@ class Client:
     async def fetch_clips(self, ids: List[str]):
         """|coro|
 
-
         Fetches clips by clip id.
         To fetch clips by user id, use :meth:`twitchio.PartialUser.fetch_clips`
 
@@ -405,13 +447,20 @@ class Client:
         data = await self._http.get_clips(ids=ids)
         return [models.Clip(self._http, d) for d in data]
 
-    async def fetch_channel(self, broadcaster: str):
-        """Retrieve channel information from the API.
+    async def fetch_channel(self, broadcaster: str, token: Optional[str] = None):
+        """|coro|
+
+        Retrieve channel information from the API.
+
+        .. note::
+            This will be deprecated in 3.0. It's recommended to use :func:`~fetch_channels` instead.
 
         Parameters
         -----------
         broadcaster: str
             The channel name or ID to request from API. Returns empty dict if no channel was found.
+        token: Optional[:class:`str`]
+            An optional OAuth token to use instead of the bot OAuth token.
 
         Returns
         --------
@@ -422,9 +471,9 @@ class Client:
             get_id = await self.fetch_users(names=[broadcaster.lower()])
             if not get_id:
                 raise IndexError("Invalid channel name.")
-            broadcaster = get_id[0].id
+            broadcaster = str(get_id[0].id)
         try:
-            data = await self._http.get_channels(broadcaster)
+            data = await self._http.get_channels(broadcaster_id=broadcaster, token=token)
 
             from .models import ChannelInfo
 
@@ -433,6 +482,27 @@ class Client:
         except HTTPException:
             raise HTTPException("Incorrect channel ID.")
 
+    async def fetch_channels(self, broadcaster_ids: List[int], token: Optional[str] = None):
+        """|coro|
+
+        Retrieve information for up to 100 channels from the API.
+
+        Parameters
+        -----------
+        broadcaster_ids: List[:class:`int`]
+            The channel ids to request from API.
+        token: Optional[:class:`str`]
+            An optional OAuth token to use instead of the bot OAuth token
+
+        Returns
+        --------
+            List[:class:`twitchio.ChannelInfo`]
+        """
+        from .models import ChannelInfo
+
+        data = await self._http.get_channels_new(broadcaster_ids=broadcaster_ids, token=token)
+        return [ChannelInfo(self._http, data=d) for d in data]
+
     async def fetch_videos(
         self,
         ids: List[int] = None,
@@ -572,7 +642,7 @@ class Client:
             game ids of streams to fetch
         user_logins: Optional[List[:class:`str`]]
             user login names of people whose streams to fetch
-        languages: Optional[List:class:`str]]
+        languages: Optional[List[:class:`str`]]
             language for the stream(s). ISO 639-1 or two letter code for supported stream language
         token: Optional[:class:`str`]
             An optional OAuth token to use instead of the bot OAuth token
@@ -667,7 +737,7 @@ class Client:
         Parameters
         -----------
         token: :class:`str`
-            An oauth token with the channel:manage:videos scope
+            An oauth token with the ``channel:manage:videos`` scope
         ids: List[:class:`int`]
             A list of video ids from the channel of the oauth token to delete
 
@@ -681,6 +751,46 @@ class Client:
 
         return resp
 
+    async def fetch_chatters_colors(self, user_ids: List[int], token: Optional[str] = None):
+        """|coro|
+
+        Fetches the color of a chatter.
+
+        Parameters
+        -----------
+        user_ids: List[:class:`int`]
+            List of user ids to fetch the colors for
+        token: Optional[:class:`str`]
+            An optional user oauth token
+
+        Returns
+        --------
+            List[:class:`twitchio.ChatterColor`]
+        """
+        data = await self._http.get_user_chat_color(user_ids, token)
+        return [models.ChatterColor(self._http, x) for x in data]
+
+    async def update_chatter_color(self, token: str, user_id: int, color: str):
+        """|coro|
+
+        Updates the color of the specified user in the specified channel/broadcaster's chat.
+
+        Parameters
+        -----------
+        token: :class:`str`
+            An oauth token with the ``user:manage:chat_color`` scope.
+        user_id: :class:`int`
+            The ID of the user whose color is being updated, this must match the user ID in the token.
+        color: :class:`str`
+            Turbo and Prime users may specify a named color or a Hex color code like #9146FF.
+            Please see the Twitch documentation for more information.
+
+        Returns
+        --------
+            None
+        """
+        await self._http.put_user_chat_color(token=token, user_id=str(user_id), color=color)
+
     async def get_webhook_subscriptions(self):
         """|coro|
 
@@ -703,7 +813,7 @@ class Client:
 
         .. note::
             This event is a callback hook. It is not a dispatched event. Any errors raised will be passed to the
-            :ref:`~event_error` event.
+            :func:`~event_error` event.
         """
         return None
 
@@ -859,6 +969,13 @@ class Client:
         """
         pass
 
+    async def event_reconnect(self):
+        """|coro|
+
+        Event called when twitch sends a RECONNECT notice.
+        The library will automatically handle reconnecting when such an event is received
+        """
+
     async def event_raw_data(self, data: str):
         """|coro|
 
@@ -879,3 +996,14 @@ class Client:
                 print(data)
         """
         pass
+
+    async def event_channel_joined(self, channel: Channel):
+        """|coro|
+
+        Event called when the bot joins a channel.
+
+        Parameters
+        ----------
+        channel: :class:`.Channel`
+            The channel that was joined.
+        """
