@@ -1,7 +1,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2017-2021 TwitchIO
+Copyright (c) 2017-present TwitchIO
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -30,6 +30,7 @@ import sys
 import traceback
 import types
 import warnings
+from functools import partial
 from typing import Callable, Optional, Union, Coroutine, Dict, List, TYPE_CHECKING, Mapping, Awaitable
 
 from twitchio.client import Client
@@ -54,10 +55,15 @@ class Bot(Client):
         client_secret: str = None,
         initial_channels: Union[list, tuple, Callable] = None,
         heartbeat: Optional[float] = 30.0,
+        retain_cache: Optional[bool] = True,
         **kwargs,
     ):
         super().__init__(
-            token=token, client_secret=client_secret, initial_channels=initial_channels, heartbeat=heartbeat
+            token=token,
+            client_secret=client_secret,
+            initial_channels=initial_channels,
+            heartbeat=heartbeat,
+            retain_cache=retain_cache,
         )
 
         self._prefix = prefix
@@ -68,7 +74,6 @@ class Bot(Client):
         else:
             self._commands = {}
             self._command_aliases = {}
-
         self._modules: Dict[str, types.ModuleType] = {}
         self._cogs: Dict[str, Cog] = {}
         self._checks: List[Callable[[Context], Union[bool, Awaitable[bool]]]] = []
@@ -99,7 +104,7 @@ class Bot(Client):
 
         Parameters
         ------------
-        client_id: :class`str`
+        client_id: :class:`str`
             Your application's Client ID.
         client_secret: :class:`str`
             An application Client Secret used to generate Access Tokens automatically.
@@ -133,6 +138,7 @@ class Bot(Client):
         self._commands = {}
         self._command_aliases = {}
         self._checks = []
+        self.registered_callbacks = {}
 
         return self
 
@@ -142,7 +148,6 @@ class Bot(Client):
         for _, obj in commands:
             if not isinstance(obj, Command):
                 continue
-
             obj._instance = self
 
             try:
@@ -159,21 +164,21 @@ class Bot(Client):
                 ret = await self._prefix(self, message)
             else:
                 ret = self._prefix(self, message)
-
         if not isinstance(ret, (list, tuple, set, str)):
             raise TypeError(f"Prefix must be of either class <list, tuple, set, str> not <{type(ret)}>")
-
         return ret
 
     async def get_prefix(self, message):
         # TODO Docs
         prefixes = await self.__get_prefixes__(message)
-
+        message_content = message.content
+        if "reply-parent-msg-id" in message.tags:
+            message_content = message_content.split(" ", 1)[1]
         if not isinstance(prefixes, str):
             for prefix in prefixes:
-                if message.content.startswith(prefix):
+                if message_content.startswith(prefix):
                     return prefix
-        elif message.content.startswith(prefixes):
+        elif message_content.startswith(prefixes):
             return prefixes
         else:
             return None
@@ -194,19 +199,16 @@ class Bot(Client):
             )
         elif not inspect.iscoroutinefunction(command._callback):
             raise TwitchCommandError(f"Failed to load command <{command.name}>. Commands must be coroutines.")
-
         self.commands[command.name] = command
 
         if not command.aliases:
             return
-
         for alias in command.aliases:
             if alias in self.commands:
                 del self.commands[command.name]
                 raise TwitchCommandError(
                     f"Failed to load command <{command.name}>, a command with that name/alias already exists."
                 )
-
             self._command_aliases[alias] = command.name
 
     def get_command(self, name: str) -> Optional[Command]:
@@ -247,7 +249,6 @@ class Bot(Client):
         for alias in list(self._command_aliases.keys()):
             if self._command_aliases[alias] == name:
                 del self._command_aliases[alias]
-
         try:
             del self._commands[name]
         except KeyError:
@@ -285,25 +286,28 @@ class Bot(Client):
         """
         if not cls:
             cls = Context
-
         prefix = await self.get_prefix(message)
         if not prefix:
             return cls(message=message, prefix=prefix, valid=False, bot=self)
-
-        content = message.content[len(prefix) : :].lstrip()  # Strip prefix and remainder whitespace
+        content = message.content
+        if "reply-parent-msg-id" in message.tags:  # Remove @username from reply message
+            content = content.split(" ", 1)[1]
+        content = content[len(prefix) : :].lstrip()  # Strip prefix and remainder whitespace
         view = StringParser()
         parsed = view.process_string(content)  # Return the string as a dict view
 
         try:
             command_ = parsed.pop(0)
         except KeyError:
-            raise CommandNotFound("No valid command was passed.")
+            context = cls(message=message, bot=self, prefix=prefix, command=None, valid=False, view=view)
+            error = CommandNotFound("No valid command was passed.")
 
+            self.run_event("command_error", context, error)
+            return context
         try:
             command_ = self._command_aliases[command_]
         except KeyError:
             pass
-
         if command_ in self.commands:
             command_ = self.commands[command_]
         else:
@@ -312,12 +316,24 @@ class Bot(Client):
 
             self.run_event("command_error", context, error)
             return context
-
         context = cls(message=message, bot=self, prefix=prefix, command=command_, valid=True, view=view)
 
         return context
 
     async def handle_commands(self, message):
+        """|coro|
+
+        This method handles commands sent from chat and invokes them.
+
+        By default, this coroutine is called within the :func:`Bot.event_message` event.
+        If you choose to override :func:`Bot.event_message` then you need to invoke this coroutine in order to handle commands.
+
+        Parameters
+        ----------
+        message: :class:`.Message`
+            The message object to get content of and context for.
+
+        """
         context = await self.get_context(message)
         await self.invoke(context)
 
@@ -325,7 +341,6 @@ class Bot(Client):
         # TODO Docs
         if not context.prefix or not context.is_valid:
             return
-
         self.run_event("command_invoke", context)
         await context.command(context)
 
@@ -339,7 +354,6 @@ class Bot(Client):
         """
         if name in self._modules:
             raise ValueError(f"Module <{name}> is already loaded")
-
         module = importlib.import_module(name)
 
         if hasattr(module, "prepare"):
@@ -348,7 +362,6 @@ class Bot(Client):
             del module
             del sys.modules[name]
             raise ImportError(f"Module <{name}> is missing a prepare method")
-
         self._modules[name] = module
 
     def unload_module(self, name: str) -> None:
@@ -361,7 +374,6 @@ class Bot(Client):
         """
         if name not in self._modules:
             raise ValueError(f"Module <{name}> is not loaded")
-
         module = self._modules.pop(name)
 
         if hasattr(module, "breakdown"):
@@ -369,15 +381,20 @@ class Bot(Client):
                 module.breakdown(self)  # type: ignore
             except:
                 pass
-
         to_delete = [cog_name for cog_name, cog in self._cogs.items() if cog.__module__ == module.__name__]
         for name in to_delete:
             self.remove_cog(name)
-
         to_delete = [name for name, cmd in self._commands.items() if cmd._callback.__module__ == module.__name__]
         for name in to_delete:
             self.remove_command(name)
-
+        to_delete = [
+            x
+            for y in self._events.items()
+            for x in y[1]
+            if isinstance(x, partial) and x.func.__module__ == module.__name__
+        ]
+        for event in to_delete:
+            self.remove_event(event)
         for m in list(sys.modules.keys()):
             if m == module.__name__ or m.startswith(module.__name__ + "."):
                 del sys.modules[m]
@@ -397,7 +414,6 @@ class Bot(Client):
         """
         if name not in self._modules:
             raise ValueError(f"Module <{name}> is not loaded")
-
         module = self._modules[name]
 
         modules = {
@@ -412,6 +428,7 @@ class Bot(Client):
         except Exception as e:
             sys.modules.update(modules)
             module.prepare(self)  # type: ignore
+            self._modules[name] = module
             raise
 
     def add_cog(self, cog: Cog):
@@ -429,10 +446,8 @@ class Bot(Client):
         """
         if not isinstance(cog, Cog):
             raise InvalidCog('Cogs must derive from "commands.Cog".')
-
         if cog.name in self._cogs:
             raise InvalidCog(f'Cog "{cog.name}" has already been loaded.')
-
         cog._load_methods(self)
         self._cogs[cog.name] = cog
 
@@ -445,7 +460,6 @@ class Bot(Client):
         """
         if cog_name not in self._cogs:
             raise InvalidCog(f"Cog '{cog_name}' not found")
-
         cog = self._cogs.pop(cog_name)
         cog._unload_methods(self)
 
@@ -536,7 +550,6 @@ class Bot(Client):
         """
         if message.echo:
             return
-
         await self.handle_commands(message)
 
     def command(
@@ -595,6 +608,5 @@ class Bot(Client):
     def check(self, func: Callable[[Context], bool]) -> Callable:
         if func in self._checks:
             raise ValueError("The function is already registered as a bot check")
-
         self._checks.append(func)
         return func
