@@ -3,7 +3,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2017-2021 TwitchIO
+Copyright (c) 2017-present TwitchIO
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -38,6 +38,24 @@ __all__ = (
 
 
 class Bucket(enum.Enum):
+    """
+    Enum values for the different cooldown buckets.
+
+    Parameters
+    ------------
+    default: :class:`enum.Enum`
+        The default bucket.
+    channel: :class:`enum.Enum`
+        Cooldown is shared amongst all chatters per channel.
+    member: :class:`enum.Enum`
+        Cooldown operates on a per channel basis per user.
+    user: :class:`enum.Enum`
+        Cooldown operates on a user basis across all channels.
+    subscriber: :class:`enum.Enum`
+        Cooldown for subscribers.
+    mod: :class:`enum.Enum`
+        Cooldown for mods.
+    """
 
     default = 0
     channel = 1
@@ -48,6 +66,41 @@ class Bucket(enum.Enum):
 
 
 class Cooldown:
+    """
+    Cooldown decorator values.
+
+    Parameters
+    ------------
+    rate: :class:`int`
+        How many times the command can be invoked before triggering a cooldown inside a time frame.
+    per: :class:`float`
+        The amount of time in seconds to wait for a cooldown when triggered.
+    bucket: :class:`Bucket`
+        The bucket that the cooldown is in.
+
+    Examples
+    ----------
+
+    .. code:: py
+
+        # Restrict a command to once every 10 seconds on a per channel basis.
+        @commands.cooldown(rate=1, per=10, bucket=commands.Bucket.channel)
+        @commands.command()
+        async def my_command(self, ctx: commands.Context):
+            pass
+
+        # Restrict a command to once every 30 seconds for each individual channel a user is in.
+        @commands.cooldown(rate=1, per=30, bucket=commands.Bucket.member)
+        @commands.command()
+        async def my_command(self, ctx: commands.Context):
+            pass
+
+        # Restrict a command to 5 times every 60 seconds globally for a user.
+        @commands.cooldown(rate=5, per=60, bucket=commands.Bucket.user)
+        @commands.command()
+        async def my_command(self, ctx: commands.Context):
+            pass
+    """
 
     __slots__ = ("_rate", "_per", "bucket", "_window", "_tokens", "_cache")
 
@@ -56,40 +109,30 @@ class Cooldown:
         self._per = per
         self.bucket = bucket
 
-        self._window = time.time() + self._per
-        self._tokens = 0
-
         self._cache = {}
 
-    def get_tokens(self, now=None):
-        now = now or time.time()
-        tokens = self._tokens
-
-        if now > self._window + self._per:
-            tokens = 0
-
-        return tokens
-
     def update_bucket(self, ctx):
         now = time.time()
 
-        self._tokens = self.get_tokens(now)
+        bucket_keys = self._bucket_keys(ctx)
+        buckets = []
 
-        if self._tokens == 0:
-            self._window = now
+        for bucket in bucket_keys:
+            (tokens, window) = self._cache[bucket]
 
-        if self._tokens == self._rate:
-            retry = self._per - (now - self._window)
-            raise CommandOnCooldown(command=ctx.command, retry_after=retry)
+            if tokens == self._rate:
+                retry = self._per - (now - window)
+                raise CommandOnCooldown(command=ctx.command, retry_after=retry)
 
-        self._tokens += 1
+            tokens += 1
 
-        if self._tokens == self._rate:
-            self._window = now
+            if tokens == self._rate:
+                window = now
+
+            self._cache[bucket] = (tokens, window)
 
     def reset(self):
-        self._tokens = 0
-        self._window = time.time()
+        self._cache = {}
 
     def _bucket_keys(self, ctx):
         buckets = []
@@ -115,7 +158,7 @@ class Cooldown:
 
     def _update_cache(self, now=None):
         now = now or time.time()
-        dead = [key for key, cooldown in self._cache.items() if now > cooldown._window + cooldown._per]
+        dead = [key for key, cooldown in self._cache.items() if now > cooldown[1] + self._per]
 
         for bucket in dead:
             del self._cache[bucket]
@@ -129,10 +172,8 @@ class Cooldown:
         buckets = []
 
         for index, bucket in enumerate(bucket_keys):
+            buckets.append(ctx.command._cooldowns[index])
             if bucket not in self._cache:
-                buckets.append(ctx.command._cooldowns[index])
-                self._cache[bucket] = ctx.command._cooldowns[index]
-            else:
-                buckets.append(self._cache[bucket])
+                self._cache[bucket] = (0, now)
 
         return buckets
