@@ -1,7 +1,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2017-2021 TwitchIO
+Copyright (c) 2017-present TwitchIO
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -39,8 +39,6 @@ if TYPE_CHECKING:
     from twitchio import Message, Chatter, PartialChatter, Channel, User, PartialUser
     from . import Cog, Bot
     from .stringparser import StringParser
-
-
 __all__ = ("Command", "command", "Group", "Context", "cooldown")
 
 
@@ -50,7 +48,6 @@ def _boolconverter(param: str):
         return True
     elif param in ("no", "n", "0", "false", "off"):
         return False
-
     raise BadArgument(f"Expected a boolean value, got {param}")
 
 
@@ -58,7 +55,6 @@ class Command:
     def __init__(self, name: str, func: Callable, **attrs) -> None:
         if not inspect.iscoroutinefunction(func):
             raise TypeError("Command callback must be a coroutine.")
-
         self._callback = func
         self._checks = []
         self._cooldowns = []
@@ -72,12 +68,10 @@ class Command:
             self._checks.extend(func.__checks__)  # type: ignore
         except AttributeError:
             pass
-
         try:
             self._cooldowns.extend(func.__cooldowns__)  # type: ignore
         except AttributeError:
             pass
-
         self.aliases = attrs.get("aliases", None)
         sig = inspect.signature(func)
         self.params = sig.parameters.copy()  # type: ignore
@@ -99,7 +93,6 @@ class Command:
     def full_name(self) -> str:
         if not self.parent:
             return self._name
-
         return f"{self.parent.full_name} {self._name}"
 
     def _resolve_converter(self, converter: Union[Callable, Awaitable, type]) -> Union[Callable[..., Any]]:
@@ -109,7 +102,6 @@ class Command:
             and converter in builtin_converter._mapping
         ):
             return builtin_converter._mapping[converter]
-
         return converter
 
     async def _convert_types(self, context: Context, param: inspect.Parameter, parsed: str) -> Any:
@@ -119,7 +111,6 @@ class Command:
                 converter = str
             else:
                 converter = type(param.default)
-
         true_converter = self._resolve_converter(converter)
 
         try:
@@ -133,14 +124,12 @@ class Command:
                 argument = await argument
         except BadArgument:
             raise
-
         except Exception as e:
             raise ArgumentParsingFailed(
                 f"Invalid argument parsed at `{param.name}` in command `{self.name}`."
                 f" Expected type {converter} got {type(parsed)}.",
                 e,
             ) from e
-
         return argument
 
     async def parse_args(self, context: Context, instance: Optional[Cog], parsed: dict, index=0) -> Tuple[list, dict]:
@@ -156,7 +145,6 @@ class Command:
                 next(iterator)
         except StopIteration:
             raise TwitchCommandError("self or ctx is a required argument which is missing.")
-
         for _, param in iterator:
             index += 1
             if param.kind == param.POSITIONAL_OR_KEYWORD:
@@ -169,36 +157,31 @@ class Command:
                 else:
                     argument = await self._convert_types(context, param, argument)
                     args.append(argument)
-
             elif param.kind == param.KEYWORD_ONLY:
                 rest = " ".join(parsed.values())
                 if rest.startswith(" "):
                     rest = rest.lstrip(" ")
-
                 if rest:
                     rest = await self._convert_types(context, param, rest)
                 elif param.default is param.empty:
                     raise MissingRequiredArgument(param)
                 else:
                     rest = param.default
-
                 kwargs[param.name] = rest
                 parsed.clear()
                 break
             elif param.VAR_POSITIONAL:
-                args.extend(parsed.values())
+                args.extend([await self._convert_types(context, param, argument) for argument in parsed.values()])
+                parsed.clear()
                 break
-
         if parsed:
             pass  # TODO Raise Too Many Arguments.
-
         return args, kwargs
 
     async def invoke(self, context: Context, *, index=0) -> None:
         # TODO Docs
         if not context.view:
             return
-
         args, kwargs = await self.parse_args(context, self._instance, context.view.words, index=index)
         context.args, context.kwargs = args, kwargs
 
@@ -216,38 +199,31 @@ class Command:
         if check_result is not True:
             context.bot.run_event("command_error", context, check_result)
             return
-
         limited = self._run_cooldowns(context)
 
         if limited:
             context.bot.run_event("command_error", context, limited[0])
             return
-
         instance = self._instance
         if instance:
             args = [instance, context]
         else:
             args = [context]
-
         await try_run(context.bot.global_before_invoke(context))
 
         if self._before_invoke:
             await try_run(self._before_invoke(*args), to_command=True)
-
         try:
             await self._callback(*args, *context.args, **context.kwargs)
         except Exception as e:
             if self.event_error:
                 await try_run(self.event_error(*args, e))
-
             context.bot.run_event("command_error", context, e)
         else:
             context.bot.run_event("command_complete", context)
-
         # Invoke our after command hooks
         if self._after_invoke:
             await try_run(self._after_invoke(*args), to_command=True)
-
         await try_run(context.bot.global_after_invoke(context))
 
     def _run_cooldowns(self, context: Context) -> Optional[List[CommandOnCooldown]]:
@@ -255,7 +231,6 @@ class Command:
             buckets = self._cooldowns[0].get_buckets(context)
         except IndexError:
             return None
-
         expired = []
 
         try:
@@ -263,7 +238,6 @@ class Command:
                 bucket.update_bucket(context)
         except CommandOnCooldown as e:
             expired.append(e)
-
         return expired
 
     async def handle_checks(self, context: Context) -> Union[Literal[True], Exception]:
@@ -273,20 +247,16 @@ class Command:
             checks = [predicate for predicate in itertools.chain(context.bot._checks, self._checks)]
         else:
             checks = self._checks
-
         try:
             for predicate in checks:
                 result = predicate(context)
 
                 if inspect.isawaitable(result):
                     result = await result  # type: ignore
-
                 if not result:
                     raise CheckFailure(f"The check {predicate} for command {self.name} failed.")
-
             if self.cog and not await self.cog.cog_check(context):
                 raise CheckFailure(f"The cog check for command <{self.name}> failed.")
-
             return True
         except Exception as e:
             return e
@@ -304,10 +274,8 @@ class Group(Command):
     async def __call__(self, context: Context, *, index=0) -> None:
         if not context.view:
             return
-
         if not context.view.words:
             return await self.invoke(context, index=index)
-
         arg: Tuple[int, str] = list(context.view.words.items())[0]  # type: ignore
         if arg[1] in self._sub_commands:
             _ctx = copy.copy(context)
@@ -317,7 +285,6 @@ class Group(Command):
 
             if self._invoke_with_subcommand:
                 await self.invoke(context, index=index)
-
         else:
             await self.invoke(context, index=index)
 
@@ -334,7 +301,6 @@ class Group(Command):
             if cmd.aliases:
                 for a in cmd.aliases:
                     self._sub_commands[a] = cmd
-
             return cmd
 
         return decorator
@@ -366,7 +332,6 @@ class Group(Command):
             if cmd.aliases:
                 for a in cmd.aliases:
                     self._sub_commands[a] = cmd
-
             return cmd
 
         return decorator
@@ -386,7 +351,6 @@ class Context(Messageable):
         self.command: Optional[Command] = attrs.get("command")
         if self.command:
             self.cog: Optional[Cog] = self.command.cog
-
         self.args: Optional[list] = attrs.get("args")
         self.kwargs: Optional[dict] = attrs.get("kwargs")
 
@@ -408,7 +372,6 @@ class Context(Messageable):
     def _bot_is_mod(self) -> bool:
         if not self.channel:
             return False
-
         cache = self._ws._cache[self.channel._name]
         for user in cache:
             if user.name == self._ws.nick:
@@ -416,7 +379,6 @@ class Context(Messageable):
                     mod = user.is_mod
                 except AttributeError:
                     return False
-
                 return mod
 
     @property
@@ -426,7 +388,6 @@ class Context(Messageable):
             users = self._ws._cache[self.channel._name]
         except (KeyError, AttributeError):
             return None
-
         return users
 
     @property
@@ -452,14 +413,47 @@ class Context(Messageable):
 
         if not self.channel:
             return None
-
         cache = self._ws._cache[self.channel._name]
         for user in cache:
             if user.name == name:
                 return user
-
         return None
 
+    async def reply(self, content: str):
+        """|coro|
+
+
+        Send a message in reply to the user who sent a message in the destination
+        associated with the dataclass.
+
+        Destination will be the context of which the message/command was sent.
+
+        Parameters
+        ------------
+        content: str
+            The content you wish to send as a message. The content must be a string.
+
+        Raises
+        --------
+        InvalidContent
+            Invalid content.
+        """
+        entity = self._fetch_channel()
+        ws = self._fetch_websocket()
+        message = self._fetch_message()
+
+        self.check_content(content)
+        self.check_bucket(channel=entity.name)
+
+        try:
+            name = entity.channel.name
+        except AttributeError:
+            name = entity.name
+        if entity.__messageable_channel__:
+            await ws.reply(message.id, f"PRIVMSG #{name} :{content}\r\n")
+        else:
+            await ws.send(f"PRIVMSG #jtv :/w {name} {content}\r\n")
+
 
 C = TypeVar("C", bound="Command")
 G = TypeVar("G", bound="Group")
