@@ -1,7 +1,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2017-2021 TwitchIO
+Copyright (c) 2017-present TwitchIO
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -35,9 +35,10 @@ __all__ = ("Cog",)
 
 
 class CogEvent:
-    def __init__(self, *, name: str, func: Callable):
+    def __init__(self, *, name: str, func: Callable, module: str):
         self.name = name
         self.func = func
+        self.module = module
 
 
 class CogMeta(type):
@@ -113,7 +114,7 @@ class Cog(metaclass=CogMeta):
                 method._instance = self
                 method.cog = self
 
-                self._commands[name] = method
+                self._commands[method.name] = method
                 bot.add_command(method)
 
         events = self._events.copy()
@@ -166,7 +167,7 @@ class Cog(metaclass=CogMeta):
         def decorator(func) -> CogEvent:
             event_name = event or func.__name__
 
-            return CogEvent(name=event_name, func=func)
+            return CogEvent(name=event_name, func=func, module=cls.__module__)
 
         return decorator
 
