@@ -1,7 +1,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2017-2021 TwitchIO
+Copyright (c) 2017-present TwitchIO
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -22,7 +22,7 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 DEALINGS IN THE SOFTWARE.
 """
 
-import datetime
+
 from typing import List, Optional
 
 from twitchio import PartialUser, Client, Channel, CustomReward, parse_timestamp
@@ -41,6 +41,7 @@ __all__ = (
     "PubSubModerationActionModeratorAdd",
     "PubSubModerationActionBanRequest",
     "PubSubModerationActionChannelTerms",
+    "PubSubChannelSubscribe",
 )
 
 
@@ -78,7 +79,7 @@ class PubSubChatMessage:
 
     def __init__(self, content: str, id: str, type: str):
         self.content = content
-        self.id = int(id)
+        self.id = id
         self.type = type
 
 
@@ -132,8 +133,8 @@ class PubSubBitsMessage(PubSubMessage):
         The amount of bits used.
     channel_id: :class:`int`
         The channel the bits were given to.
-    user: :class:`twitchio.PartialUser`
-        The user giving the bits.
+    user: Optional[:class:`twitchio.PartialUser`]
+        The user giving the bits. Can be None if anonymous.
     version: :class:`str`
         The event version.
     """
@@ -143,16 +144,21 @@ class PubSubBitsMessage(PubSubMessage):
     def __init__(self, client: Client, topic: str, data: dict):
         super().__init__(client, topic, data)
 
-        self.message = PubSubChatMessage(data["chat_message"], data["message_id"], data["message_type"])
+        data = data["message"]
+        self.message = PubSubChatMessage(data["data"]["chat_message"], data["message_id"], data["message_type"])
         self.badge_entitlement = (
-            PubSubBadgeEntitlement(data["badge_entitlement"]["new_version"], data["badge_entitlement"]["old_version"])
-            if data["badge_entitlement"]
+            PubSubBadgeEntitlement(
+                data["data"]["badge_entitlement"]["new_version"], data["data"]["badge_entitlement"]["old_version"]
+            )
+            if data["data"]["badge_entitlement"]
             else None
         )
-        self.bits_used: int = data["bits_used"]
-        self.channel_id: int = int(data["channel_id"])
+        self.bits_used: int = data["data"]["bits_used"]
+        self.channel_id: int = int(data["data"]["channel_id"])
         self.user = (
-            PartialUser(client._http, data["user_id"], data["user_name"]) if data["user_id"] is not None else None
+            PartialUser(client._http, data["data"]["user_id"], data["data"]["user_name"])
+            if data["data"]["user_id"]
+            else None
         )
         self.version: str = data["version"]
 
@@ -179,6 +185,7 @@ class PubSubBitsBadgeMessage(PubSubMessage):
 
     def __init__(self, client: Client, topic: str, data: dict):
         super().__init__(client, topic, data)
+        data = data["message"]
         self.user = PartialUser(client._http, data["user_id"], data["user_name"])
         self.channel: Channel = client.get_channel(data["channel_name"]) or Channel(
             name=data["channel_name"], websocket=client._connection
@@ -238,7 +245,7 @@ class PubSubModerationAction(PubSubMessage):
         The arguments given to the command.
     created_by: :class:`twitchio.PartialUser`
         The user that created the action.
-    message_id: :class:`str`
+    message_id: Optional[:class:`str`]
         The id of the message that created this action.
     target: :class:`twitchio.PartialUser`
         The target of this action.
@@ -255,7 +262,7 @@ class PubSubModerationAction(PubSubMessage):
         self.created_by = PartialUser(
             client._http, data["message"]["data"]["created_by_user_id"], data["message"]["data"]["created_by"]
         )
-        self.message_id: str = data["message"]["data"]["msg_id"]
+        self.message_id: Optional[str] = data["message"]["data"].get("msg_id")
         self.target = (
             PartialUser(
                 client._http, data["message"]["data"]["target_user_id"], data["message"]["data"]["target_user_login"]
@@ -263,7 +270,7 @@ class PubSubModerationAction(PubSubMessage):
             if data["message"]["data"]["target_user_id"]
             else None
         )
-        self.from_automod: bool = data["message"]["data"]["from_automod"]
+        self.from_automod: bool = data["message"]["data"].get("from_automod", False)
 
 
 class PubSubModerationActionBanRequest(PubSubMessage):
@@ -280,11 +287,9 @@ class PubSubModerationActionBanRequest(PubSubMessage):
         The user that created the action.
     target: :class:`twitchio.PartialUser`
         The target of this action.
-    from_automod: :class:`bool`
-        Whether this action was done automatically or not.
     """
 
-    __slots__ = "action", "args", "created_by", "message_id", "target", "from_automod"
+    __slots__ = "action", "args", "created_by", "message_id", "target"
 
     def __init__(self, client: Client, topic: str, data: dict):
         super().__init__(client, topic, data)
@@ -320,7 +325,7 @@ class PubSubModerationActionChannelTerms(PubSubMessage):
         The requester of this Term.
     """
 
-    __slots__ = "type", "channel_id", "id", "text", "requester", "expires_at", "updated_at", "from_automod"
+    __slots__ = "type", "channel_id", "id", "text", "requester", "expires_at", "updated_at"
 
     def __init__(self, client: Client, topic: str, data: dict):
         super().__init__(client, topic, data)
@@ -332,12 +337,107 @@ class PubSubModerationActionChannelTerms(PubSubMessage):
             client._http, data["message"]["data"]["requester_id"], data["message"]["data"]["requester_login"]
         )
 
-        self.expires_at = self.updated_at = None
-        if data["message"]["data"]["expires_at"]:
-            self.expires_at = parse_timestamp(data["message"]["data"]["expires_at"])
+        self.expires_at = (
+            parse_timestamp(data["message"]["data"]["expires_at"]) if data["message"]["data"]["expires_at"] else None
+        )
+        self.updated_at = (
+            parse_timestamp(data["message"]["data"]["updated_at"]) if data["message"]["data"]["updated_at"] else None
+        )
 
-        if data["message"]["data"]["updated_at"]:
-            self.updated_at = parse_timestamp(data["message"]["data"]["expires_at"])
+
+class PubSubChannelSubscribe(PubSubMessage):
+    """
+    Channel subscription
+
+    Attributes
+    -----------
+    channel: :class:`twitchio.Channel`
+        Channel that has been subscribed or subgifted.
+    context: :class:`str`
+        Event type associated with the subscription product.
+    user: Optional[:class:`twitchio.PartialUser`]
+        The person who subscribed or sent a gift subscription. Can be None if anonymous.
+    message: :class:`str`
+        Message sent with the sub/resub.
+    emotes: Optional[List[:class:`dict`]]
+        Emotes sent with the sub/resub.
+    is_gift: :class:`bool`
+        If this sub message was caused by a gift subscription.
+    recipient: Optional[:class:`twitchio.PartialUser`]
+        The person the who received the gift subscription.
+    sub_plan: :class:`str`
+        Subscription Plan ID.
+    sub_plan_name: :class:`str`
+        Channel Specific Subscription Plan Name.
+    time: :class:`datetime.datetime`
+        Time when the subscription or gift was completed. RFC 3339 format.
+    cumulative_months: :class:`int`
+        Cumulative number of tenure months of the subscription.
+    streak_months: Optional[:class:`int`]
+        Denotes the user's most recent (and contiguous) subscription tenure streak in the channel.
+    multi_month_duration: Optional[:class:`int`]
+        Number of months gifted as part of a single, multi-month gift OR number of months purchased as part of a multi-month subscription.
+    """
+
+    __slots__ = (
+        "channel",
+        "context",
+        "user",
+        "message",
+        "emotes",
+        "is_gift",
+        "recipient",
+        "sub_plan",
+        "sub_plan_name",
+        "time",
+        "cumulative_months",
+        "streak_months",
+        "multi_month_duration",
+    )
+
+    def __init__(self, client: Client, topic: str, data: dict):
+        super().__init__(client, topic, data)
+
+        subscription = data["message"]
+
+        self.channel: Channel = client.get_channel(subscription["channel_name"]) or Channel(
+            name=subscription["channel_name"], websocket=client._connection
+        )
+        self.context: str = subscription["context"]
+        try:
+            self.user = PartialUser(client._http, int(subscription["user_id"]), subscription["user_name"])
+        except KeyError:
+            self.user = None
+
+        self.message: str = subscription["sub_message"]["message"]
+        try:
+            self.emotes = subscription["sub_message"]["emotes"]
+        except KeyError:
+            self.emotes = None
+
+        self.is_gift: bool = subscription["is_gift"]
+        try:
+            self.recipient = PartialUser(
+                client._http, int(subscription["recipient_id"]), subscription["recipient_user_name"]
+            )
+        except KeyError:
+            self.recipient = None
+
+        self.sub_plan: str = subscription["sub_plan"]
+        self.sub_plan_name: str = subscription["sub_plan_name"]
+        self.time = parse_timestamp(subscription["time"])
+        try:
+            self.cumulative_months = int(subscription["cumulative_months"])
+        except KeyError:
+            self.cumulative_months = None
+        try:
+            self.streak_months = int(subscription["streak_months"])
+        except KeyError:
+            self.streak_months = None
+        try:
+            self.multi_month_duration = int(subscription["multi_month_duration"])
+        except KeyError:
+            self.multi_month_duration = None
 
 
 class PubSubModerationActionModeratorAdd(PubSubMessage):
@@ -391,7 +491,7 @@ def _find_mod_action(client: Client, topic: str, data: dict):
 _mapping = {
     "channel-bits-events-v2": ("pubsub_bits", PubSubBitsMessage),
     "channel-bits-badge-unlocks": ("pubsub_bits_badge", PubSubBitsBadgeMessage),
-    "channel-subscribe-events-v1": ("pubsub_subscription", None),
+    "channel-subscribe-events-v1": ("pubsub_subscription", PubSubChannelSubscribe),
     "chat_moderator_actions": ("pubsub_moderation", _find_mod_action),
     "channel-points-channel-v1": ("pubsub_channel_points", PubSubChannelPointsMessage),
     "whispers": ("pubsub_whisper", None),
