@@ -1,7 +1,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2017-2021 TwitchIO
+Copyright (c) 2017-present TwitchIO
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -32,8 +32,26 @@ if TYPE_CHECKING:
 
 
 class Message:
+    """
+    Attributes
+    -----------
+    content: :class:`str`
+        The content of this message.
+    echo: :class:`bool`
+        Boolean representing if this is a self-message or not.
 
-    __slots__ = ("_raw_data", "content", "_author", "echo", "_timestamp", "_channel", "_tags", "_id")
+    """
+
+    __slots__ = (
+        "_raw_data",
+        "content",
+        "_author",
+        "echo",
+        "_timestamp",
+        "_channel",
+        "_tags",
+        "_id",
+    )
 
     def __init__(self, **kwargs):
         self._raw_data = kwargs.get("raw_data")
@@ -48,7 +66,7 @@ class Message:
             self._timestamp = self._tags["tmi-sent-ts"]
         except KeyError:
             self._id = None
-            self._timestamp = time.time()
+            self._timestamp = datetime.datetime.now().timestamp() * 1000
 
     @property
     def id(self) -> str:
