@@ -3,7 +3,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2017-2021 TwitchIO
+Copyright (c) 2017-present TwitchIO
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -44,8 +44,6 @@ from .chatter import Chatter, PartialChatter, WhisperChatter
 
 if TYPE_CHECKING:
     from .client import Client
-
-
 log = logging.getLogger(__name__)
 HOST = "wss://irc-ws.chat.twitch.tv:443"
 
@@ -60,6 +58,7 @@ class WSConnection:
         token: str = None,
         modes: tuple = None,
         initial_channels: List[str] = None,
+        retain_cache: Optional[bool] = True,
     ):
         self._loop = loop
         self._backoff = ExponentialBackoff()
@@ -90,9 +89,11 @@ class WSConnection:
         }
 
         self.nick = None
+        self.user_id = None
         self._token = token
         self.modes = modes or ("commands", "tags", "membership")
         self._initial_channels = initial_channels or []
+        self._retain_cache = retain_cache
 
         if callable(self._initial_channels):
             _temp_initial_channels = self._initial_channels()
@@ -100,7 +101,6 @@ class WSConnection:
                 self._initial_channels = _temp_initial_channels
             else:
                 self._initial_channels = [_temp_initial_channels]
-
         self._last_ping = 0
         self._reconnect_requested = False
 
@@ -119,14 +119,12 @@ class WSConnection:
 
         if self._keeper:
             self._keeper.cancel()  # Stop our current keep alive.
-
         if self.is_alive:
             await self._websocket.close()  # If for some reason we are in a weird state, close it before retrying.
-
         if not self._client._http.nick:
             data = await self._client._http.validate(token=self._token)
             self.nick = data["login"]
-
+            self.user_id = int(data["user_id"])
         session = self._client._http.session
 
         try:
@@ -137,12 +135,10 @@ class WSConnection:
 
             await asyncio.sleep(retry)
             return asyncio.create_task(self._connect())
-
         if time.time() > self._last_ping + 240 or self._reconnect_requested:
             # Re-authenticate as we have surpassed a PING request or Twitch issued a RECONNECT.
             await self.authenticate(self._initial_channels)
             self._reconnect_requested = False
-
         self._keeper = asyncio.create_task(self._keep_alive())  # Create our keep alive.
         self._ws_ready_event.set()
 
@@ -152,14 +148,12 @@ class WSConnection:
 
         if not self._last_ping:
             self._last_ping = time.time()
-
-        while not self._websocket.closed:
+        while not self._websocket.closed and not self._reconnect_requested:
             msg = await self._websocket.receive()  # Receive data...
 
             if msg.type is aiohttp.WSMsgType.CLOSED:
                 log.error(f"Websocket connection was closed: {msg.extra}")
                 break
-
             data = msg.data
             if data:
                 log.debug(f" < {data}")
@@ -169,7 +163,6 @@ class WSConnection:
                 for event in events:
                     if not event:
                         continue
-
                     task = asyncio.create_task(self._process_data(event))
                     task.add_done_callback(partial(self._task_callback, event))  # Process our raw data
 
@@ -197,7 +190,6 @@ class WSConnection:
 
             task = asyncio.create_task(self._process_data(dummy))
             task.add_done_callback(partial(self._task_callback, dummy))  # Process our raw data
-
         await self._websocket.send_str(message + "\r\n")
 
     async def reply(self, msg_id: str, message: str):
@@ -212,7 +204,6 @@ class WSConnection:
             dummy = f"> @reply-parent-msg-id={msg_id} :{self.nick}!{self.nick}@{self.nick}.tmi.twitch.tv PRIVMSG(ECHO) #{channel} {content}\r\n"
             task = asyncio.create_task(self._process_data(dummy))
             task.add_done_callback(partial(self._task_callback, dummy))  # Process our raw data
-
         await self._websocket.send_str(f"@reply-parent-msg-id={msg_id} {message} \r\n")
 
     async def authenticate(self, channels: Union[list, tuple]):
@@ -232,19 +223,43 @@ class WSConnection:
         """
         if not self.is_alive:
             return
-
         await self.send(f"PASS oauth:{self._token}\r\n")
         await self.send(f"NICK {self.nick}\r\n")
 
         for cap in self.modes:
             await self.send(f"CAP REQ :twitch.tv/{cap}")  # Ideally no one should overwrite defaults...
-
         if not channels and not self._initial_channels:
             return
-
         channels = channels or self._initial_channels
         await self.join_channels(*channels)
 
+    async def part_channels(self, *channels: str):
+        """|coro|
+
+        Attempt to part the provided channels.
+
+        Parameters
+        ----------
+        *channels: str
+            An argument list of channels to attempt parting.
+        """
+
+        for channel in channels:
+            channel = re.sub("[#]", "", channel).lower()
+            await self.send(f"PART #{channel}\r\n")
+            if self._retain_cache:
+                self._cache.pop(channel, None)
+
+    def _assign_timeout(self, channel_count: int):
+        if channel_count <= 40:
+            return 30
+        elif channel_count <= 60:
+            return 40
+        elif channel_count <= 80:
+            return 50
+        else:
+            return 60
+
     async def join_channels(self, *channels: str):
         """|coro|
 
@@ -255,29 +270,29 @@ class WSConnection:
         *channels : str
             An argument list of channels to attempt joining.
         """
-        async with self._join_lock:  # acquire a lock, allowing only one join_channels at once...
-            for channel in channels:
-                if self._join_handle < time.time():  # Handle is less than the current time
-                    self._join_tick = 20  # So lets start a new rate limit bucket..
-                    self._join_handle = time.time() + 10  # Set the handle timeout time
+        async with self._join_lock:
+            channel_count = len(channels)
+            if channel_count > 20:
+                timeout = self._assign_timeout(channel_count)
+                chunks = [channels[i : i + 20] for i in range(0, len(channels), 20)]
+                for chunk in chunks:
+                    for channel in chunk:
+                        asyncio.create_task(self._join_channel(channel, timeout))
+                    await asyncio.sleep(11)
+            else:
+                for channel in channels:
+                    asyncio.create_task(self._join_channel(channel, 11))
 
-                if self._join_tick == 0:  # We have exhausted the bucket, wait so we can make a new one...
-                    await asyncio.sleep(self._join_handle - time.time())
-                    continue
-
-                asyncio.create_task(self._join_channel(channel))
-                self._join_tick -= 1
-
-    async def _join_channel(self, entry):
+    async def _join_channel(self, entry: str, timeout: int):
         channel = re.sub("[#]", "", entry).lower()
         await self.send(f"JOIN #{channel}\r\n")
 
         self._join_pending[channel] = fut = self._loop.create_future()
-        asyncio.create_task(self._join_future_handle(fut, channel))
+        asyncio.create_task(self._join_future_handle(fut, channel, timeout))
 
-    async def _join_future_handle(self, fut: asyncio.Future, channel: str):
+    async def _join_future_handle(self, fut: asyncio.Future, channel: str, timeout: int):
         try:
-            await asyncio.wait_for(fut, timeout=10)
+            await asyncio.wait_for(fut, timeout=timeout)
         except asyncio.TimeoutError:
             log.error(f'The channel "{channel}" was unable to be joined. Check the channel is valid.')
             self._join_pending.pop(channel)
@@ -302,7 +317,6 @@ class WSConnection:
                 f'Login unsuccessful with token "{self._token}". ' f'Check your scopes for "chat_login" and try again.'
             )
             return await self._close()
-
         partial_ = self._actions.get(parsed["action"])
         if partial_:
             await partial_(parsed)
@@ -315,10 +329,8 @@ class WSConnection:
                 fut.exception()
             except asyncio.InvalidStateError:
                 pass
-
             if fut.done():
                 futures.remove(fut)
-
         if futures:
             await asyncio.wait(futures)
 
@@ -330,24 +342,26 @@ class WSConnection:
             await self.is_ready.wait()
             self.dispatch("ready")
             self._init = True
-
         elif code == 353:
             if parsed["channel"] == "TWITCHIOFAILURE":
                 self._initial_channels.remove(parsed["batches"][0])
-
-            if parsed["channel"] in self._initial_channels and not self._init:
+            if parsed["channel"] in [c.lower().lstrip("#") for c in self._initial_channels] and not self._init:
                 self._join_load[parsed["channel"]] = None
-
             if len(self._join_load) == len(self._initial_channels):
                 for channel in self._initial_channels:
-                    self._join_load.pop(channel)
+                    self._join_load.pop(channel.lower().lstrip("#"))
                     self._cache_add(parsed)
                 self.is_ready.set()
             else:
                 self._cache_add(parsed)
+        elif code in {2, 3, 4, 366, 372, 375}:
+            return
+        elif code == 376:
+            if not self._initial_channels:
+                self.is_ready.set()
+            return
         elif self.is_ready.is_set():
             return
-
         else:
             self.is_ready.set()
             # self.dispatch("ready")
@@ -359,6 +373,24 @@ class WSConnection:
 
     async def _part(self, parsed):  # TODO
         log.debug(f'ACTION: PART:: {parsed["channel"]}')
+        channel = parsed["channel"]
+
+        if self._join_pending:
+            try:
+                self._join_pending[channel].set_result(None)
+            except KeyError:
+                pass
+            else:
+                self._join_pending.pop(channel)
+        if not self._retain_cache:
+            self._cache.pop(channel, None)
+        channel = Channel(name=channel, websocket=self)
+        user = Chatter(name=parsed["user"], bot=self._client, websocket=self, channel=channel, tags=parsed["badges"])
+        try:
+            self._cache[channel.name].discard(user)
+        except KeyError:
+            pass
+        self.dispatch("part", user)
 
     async def _privmsg(self, parsed):  # TODO(Update Cache properly)
         log.debug(f'ACTION: PRIVMSG:: {parsed["channel"]}')
@@ -373,7 +405,6 @@ class WSConnection:
             user = Chatter(
                 tags=parsed["badges"], name=parsed["user"], channel=channel, bot=self._client, websocket=self
             )
-
         message = Message(
             raw_data=parsed["data"],
             content=parsed["message"],
@@ -425,13 +456,13 @@ class WSConnection:
                 pass
             else:
                 self._join_pending.pop(channel)
-
         if parsed["user"] != self._client.nick:
             self._cache_add(parsed)
-
         channel = Channel(name=channel, websocket=self)
         user = Chatter(name=parsed["user"], bot=self._client, websocket=self, channel=channel, tags=parsed["badges"])
 
+        if user.name == self._client.nick:
+            self.dispatch("channel_joined", channel)
         self.dispatch("join", channel, user)
 
     def _cache_add(self, parsed: dict):
@@ -439,7 +470,6 @@ class WSConnection:
 
         if channel not in self._cache:
             self._cache[channel] = set()
-
         channel_ = Channel(name=channel, websocket=self)
 
         if parsed["batches"]:
@@ -458,6 +488,9 @@ class WSConnection:
     async def _reconnect(self, parsed):
         log.debug("ACTION: RECONNECT:: Twitch has gracefully closed the connection and will reconnect.")
         self._reconnect_requested = True
+        self._keeper.cancel()
+        self._loop.create_task(self._connect())
+        self.dispatch("reconnect")
 
     def dispatch(self, event: str, *args, **kwargs):
         log.debug(f"Dispatching event: {event}")
@@ -482,6 +515,7 @@ class WSConnection:
 
         for fut in futures:
             fut.cancel()
-
-        await self._websocket.close()
-        self._loop.stop()
+        if self._websocket:
+            await self._websocket.close()
+        if self._client._http.session:
+            await self._client._http.session.close()
